@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.cpp,v 1.208 2006/09/16 17:11:41 vkurland Exp $
+  $Id: FWWindow.cpp,v 1.211 2006/10/22 04:39:36 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -121,6 +121,7 @@
 #include <qfile.h>
 #include <qfiledialog.h>
 #include <qpixmap.h>
+#include <qpixmapcache.h>
 #include <qheader.h>
 #include <qtabwidget.h>
 #include <qcombobox.h>
@@ -494,6 +495,8 @@ void FWWindow::fileNew()
 
 void FWWindow::fileOpen()
 {    
+    if (fwbdebug) qDebug("FWWindow::fileOpen(): start");
+
     RCSFileDialog   fd(this, 0, true);
 
     if ( fd.exec()== QDialog::Accepted )
@@ -1709,8 +1712,13 @@ void FWWindow::addFirewallToList(FWObject *o)
         fwList->removeItem(0);
     }
 
-    fwList->insertItem( QPixmap::fromMimeSource( icn_filename ),
-                        QString::fromUtf8(o->getName().c_str()) );
+    QPixmap pm;
+    if ( ! QPixmapCache::find( icn_filename, pm) )
+    {
+        pm = QPixmap::fromMimeSource( icn_filename );
+        QPixmapCache::insert( icn_filename, pm);
+    }
+    fwList->insertItem( pm, QString::fromUtf8(o->getName().c_str()) );
 
     firewalls.push_back(o);
 
@@ -1787,7 +1795,13 @@ void FWWindow::updateFirewallName(FWObject *obj,const QString &oldName)
     {
         if ( (*i)->getId()==obj->getId())
         {
-            fwList->changeItem(  QPixmap::fromMimeSource( icn_filename ),
+            QPixmap pm;
+            if ( ! QPixmapCache::find( icn_filename, pm) )
+            {
+                pm = QPixmap::fromMimeSource( icn_filename );
+                QPixmapCache::insert( icn_filename, pm);
+            }
+            fwList->changeItem(  pm,
                                  QString::fromUtf8(obj->getName().c_str()),
                                  n );
             if (n==fwList->currentItem ())
@@ -2294,6 +2308,7 @@ void FWWindow::search()
     auxiliaryPanel->show();
     findObjectWidget->show();
 }
+
 void FWWindow::findWhereUsed(FWObject * obj)
 {
     findObjectWidget->hide();
