@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: InterfaceDialog.cpp,v 1.23 2005/03/21 06:36:47 vkurland Exp $
+  $Id: InterfaceDialog.cpp,v 1.27 2006/05/13 06:53:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -51,6 +51,7 @@
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
 #include <qpushbutton.h>
+#include <qlabel.h>
 
 using namespace std;
 using namespace libfwbuilder;
@@ -58,8 +59,10 @@ using namespace libfwbuilder;
 InterfaceDialog::InterfaceDialog(QWidget *parent) :
     InterfaceDialog_q(parent)
 {
-    seclevelFrame->hide();
-    netzoneFrame->hide();
+    seclevel->hide();
+    seclevelLabel->hide();
+    netzone->hide();
+    netzoneLabel->hide();
 
     obj=NULL;
 }
@@ -77,9 +80,12 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
     label->setText( QString::fromUtf8(s->getLabel().c_str()) );
 
-    regular->setChecked( ! s->isDyn() && ! s->isUnnumbered() );
+    regular->setChecked( ! s->isDyn() &&
+                         ! s->isUnnumbered() &&
+                         ! s->isBridgePort() );
     dynamic->setChecked( s->isDyn() );
     unnumbered->setChecked( s->isUnnumbered() );
+    bridgeport->setChecked( s->isBridgePort() );
 
     management->setChecked( s->isManagement() );
 
@@ -96,8 +102,10 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     {
         ext->setEnabled( false );
         ext->hide();
-        seclevelFrame->hide();
-        netzoneFrame->hide();
+        seclevel->hide();
+        seclevelLabel->hide();
+        netzone->hide();
+        netzoneLabel->hide();
     }
 
     bool supports_security_levels=false;
@@ -115,19 +123,22 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     {
         if (supports_security_levels)
         {
-            seclevelFrame->show();
+            seclevel->show();
+            seclevelLabel->show();
             ext->hide();
             seclevel->setValue( obj->getInt("security_level") );
         } else
         {
-            seclevelFrame->hide();
+            seclevel->hide();
+            seclevelLabel->hide();
             ext->show();
             ext->setChecked( obj->getInt("security_level")==0 );
         }
 
         if (supports_network_zones)
         {
-            netzoneFrame->show();
+            netzone->show();
+            netzoneLabel->show();
 
             netzoneObjectIDs.clear();
             netzoneObjectNos.clear();
@@ -199,10 +210,13 @@ void InterfaceDialog::loadFWObject(FWObject *o)
             netzone->setCurrentItem( netzoneObjectIDs[id] );
         }
         else
-            netzoneFrame->hide();
+        {
+            netzone->hide();
+            netzoneLabel->hide();
+        }
     }
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
 
     obj_name->setEnabled(!o->isReadOnly());
     setDisabledPalette(obj_name);
@@ -225,6 +239,9 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     unnumbered->setEnabled(!o->isReadOnly());
     setDisabledPalette(unnumbered);
 
+    bridgeport->setEnabled(!o->isReadOnly());
+    setDisabledPalette(bridgeport);
+
     management->setEnabled(!o->isReadOnly());
     setDisabledPalette(management);
 
@@ -243,7 +260,8 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     
 void InterfaceDialog::changed()
 {
-    apply->setEnabled( true );
+    //apply->setEnabled( true );
+    emit changed_sign();
 }
 
 void InterfaceDialog::validate(bool *res)
@@ -255,7 +273,7 @@ void InterfaceDialog::validate(bool *res)
 
 void InterfaceDialog::isChanged(bool *res)
 {
-    *res=(!init && apply->isEnabled());
+    //*res=(!init && apply->isEnabled());
 }
 
 void InterfaceDialog::libChanged()
@@ -279,6 +297,7 @@ void InterfaceDialog::applyChanges()
     s->setLabel( string(label->text().utf8()) );
     s->setDyn( dynamic->isChecked() );
     s->setUnnumbered( unnumbered->isChecked() );
+    s->setBridgePort( bridgeport->isChecked() );
 
     FWObject *f = obj->getParent();
     bool supports_security_levels =false;
@@ -310,7 +329,8 @@ void InterfaceDialog::applyChanges()
                       QString::fromUtf8(oldname.c_str()),
                       QString::fromUtf8(oldlabel.c_str()));
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
 }
 
 void InterfaceDialog::discardChanges()
