@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PhysicalAddressDialog.cpp,v 1.16 2004/06/17 06:37:01 vkurland Exp $
+  $Id: PhysicalAddressDialog.cpp,v 1.19 2006/05/13 06:53:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -66,7 +66,7 @@ void PhysicalAddressDialog::loadFWObject(FWObject *o)
 
     libs->setEnabled( false );
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
 
     obj_name->setEnabled(!o->isReadOnly());
     setDisabledPalette(obj_name);
@@ -86,7 +86,8 @@ void PhysicalAddressDialog::loadFWObject(FWObject *o)
     
 void PhysicalAddressDialog::changed()
 {
-    apply->setEnabled( true );
+    //apply->setEnabled( true );
+    emit changed_sign();
 }
 
 void PhysicalAddressDialog::validate(bool *res)
@@ -96,7 +97,7 @@ void PhysicalAddressDialog::validate(bool *res)
 
 void PhysicalAddressDialog::isChanged(bool *res)
 {
-    *res=(!init && apply->isEnabled());
+    //*res=(!init && apply->isEnabled());
 }
 
 void PhysicalAddressDialog::libChanged()
@@ -127,7 +128,8 @@ void PhysicalAddressDialog::applyChanges()
 
     init=false;
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
+    om->updateLastModifiedTimestampForAllFirewalls(obj);
 }
 
 void PhysicalAddressDialog::discardChanges()
