@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: listOfLibraries.cpp,v 1.17 2005/03/21 06:36:47 vkurland Exp $
+  $Id: listOfLibraries.cpp,v 1.18 2006/10/21 06:19:36 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -58,7 +58,8 @@ listOfLibraries::listOfLibraries()
     add(tempfname.c_str());
 
 #ifdef _WIN32
-    QString dir = appRootDir + "/lib";
+    string ts = appRootDir + "/lib";
+    QString dir = ts.c_str();
 #else
     QString dir = QString(getenv("HOME")) + "/.fwbuilder/lib";
 #endif
