@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: MangleTableCompiler_ipt.cpp,v 1.5 2006/06/15 05:06:07 vkurland Exp $
+  $Id: MangleTableCompiler_ipt.cpp,v 1.7 2006/09/09 05:05:00 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -60,12 +60,24 @@ int MangleTableCompiler_ipt::prolog()
 bool MangleTableCompiler_ipt::keepMangleTableRules::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
+    FWOptions *ruleopt =rule->getOptionsObject();
 
-    if (rule->getAction() != PolicyRule::Tag &&
-        rule->getAction() != PolicyRule::Route &&
-        rule->getAction() != PolicyRule::Classify) return true;
+    if (rule->getAction() == PolicyRule::Branch &&
+        ruleopt->getBool("ipt_branch_in_mangle"))
+    {
+        PolicyRule* r = 
+            PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+        compiler->temp_ruleset->add(r);
+        r->duplicate(rule);
+        r->setStr("ipt_chain","PREROUTING");
+        tmp_queue.push_back(r);
 
-    tmp_queue.push_back(rule);
+        tmp_queue.push_back(rule);
+    }
+
+    if (rule->getAction() == PolicyRule::Tag ||
+        rule->getAction() == PolicyRule::Route ||
+        rule->getAction() == PolicyRule::Classify) tmp_queue.push_back(rule);
 
     return true;
 }
