@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_PrintRule.cpp,v 1.11.4.1 2006/03/19 06:08:18 vkurland Exp $
+  $Id: PolicyCompiler_PrintRule.cpp,v 1.30 2006/06/15 05:06:08 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,6 +24,7 @@
 */
 
 #include "PolicyCompiler_ipt.h"
+#include "OSConfigurator_linux24.h"
 
 #include "fwbuilder/RuleElement.h"
 #include "fwbuilder/IPService.h"
@@ -31,8 +32,10 @@
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/CustomService.h"
+#include "fwbuilder/TagService.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Network.h"
+#include "fwbuilder/DNSName.h"
 
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/RuleElement.h"
@@ -41,6 +44,7 @@
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Resources.h"
+#include "fwbuilder/AddressTable.h"
 
 #include "combinedAddress.h"
 
@@ -76,9 +80,13 @@ using namespace std;
 string PolicyCompiler_ipt::PrintRule::_createChain(const string &chain)
 {
     string res;
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
+
     if ( ! chains[chain] )
     {
-	res = "$IPTABLES -N " + chain + "\n";
+	res = "$IPTABLES -N " + chain;
+        if (ipt_comp->my_table != "filter") res += " -t " + ipt_comp->my_table;
+        res += "\n";
 	chains[chain]=true;
     }
     return res;
@@ -86,7 +94,13 @@ string PolicyCompiler_ipt::PrintRule::_createChain(const string &chain)
 
 string PolicyCompiler_ipt::PrintRule::_startRuleLine()
 {            
-    return string("$IPTABLES -A ");
+    string res = "$IPTABLES ";
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
+
+    if (ipt_comp->my_table != "filter") res += "-t " + ipt_comp->my_table + " ";
+
+    res += "-A ";
+    return res;
 }
 
 string PolicyCompiler_ipt::PrintRule::_endRuleLine()
@@ -144,17 +158,16 @@ string PolicyCompiler_ipt::PrintRule::_printChain(PolicyRule *rule)
     return s;
 }
 
-string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
+string PolicyCompiler_ipt::PrintRule::_printModules(PolicyRule *rule)
 {
     std::ostringstream ostr;
 
     string target=rule->getStr("ipt_target");
     if (target.empty()) target="UNKNOWN";
-    int lim;
 
-    if ( compiler->getCachedFwOpt()->getBool("use_ULOG") &&
-         target=="LOG") target="ULOG";
-    
+    FWOptions *ruleopt =rule->getOptionsObject();
+    int lim = 0;
+
 /*
  * Here is what do we do with limits:
  *
@@ -166,14 +179,12 @@ string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
  * 
  *    this is so as of 1.0.11 ( 28/06/03 )  --vk
  */
-    FWOptions *ruleopt =rule->getOptionsObject();
-
-    if (target=="LOG" || target=="ULOG")    
+    if (target=="LOG")
     {
         FWOptions *compopt=compiler->getCachedFwOpt();
         if ((lim=compopt->getInt("limit_value"))>0)
         {
-            ostr << "  -m limit --limit " << lim;
+            ostr << " -m limit --limit " << lim;
         
             string ls=compopt->getStr("limit_suffix");
             if (!ls.empty()) ostr << ls;
@@ -184,7 +195,7 @@ string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
     } else {
         if (ruleopt!=NULL && (lim=ruleopt->getInt("limit_value"))>0)
         {
-            ostr << "  -m limit --limit " << lim;
+            ostr << " -m limit --limit " << lim;
 
             string ls=ruleopt->getStr("limit_suffix");
             if (!ls.empty()) ostr << ls;
@@ -194,6 +205,58 @@ string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
         }
     }
 
+    if (ruleopt!=NULL && (lim=ruleopt->getInt("connlimit_value"))>0)
+    {
+        ostr << " -m connlimit --connlimit-above " << lim;
+
+        int ml=ruleopt->getInt("connlimit_masklen");
+        if (ml>0) ostr << " --connlimit-mask " << ml;
+    }
+
+    if (ruleopt!=NULL && (lim=ruleopt->getInt("hashlimit_value"))>0)
+    {
+        string module_name = "hashlimit";
+        if (ruleopt->getBool("hashlimit_dstlimit"))
+            module_name = "dstlimit";
+
+        ostr << " -m " << module_name << " --" << module_name << " " << lim;
+
+        string ls=ruleopt->getStr("hashlimit_suffix");
+        if (!ls.empty()) ostr << ls;
+            
+        int lb=ruleopt->getInt("hashlimit_burst");
+        if (lb>0) ostr << " --" << module_name << "-burst " << lb;
+
+        ls=ruleopt->getStr("hashlimit_mode");
+        if (!ls.empty()) ostr << " --" << module_name << "-mode " << ls;
+
+        std::ostringstream hn;
+        hn << "htable_rule_" << rule->getPosition();
+        ostr << " --" << module_name << "-name " << hn.str();
+    }
+
+    return ostr.str();
+}
+
+
+string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
+{
+    std::ostringstream ostr;
+
+    string target=rule->getStr("ipt_target");
+    if (target.empty()) target="UNKNOWN";
+
+    FWOptions *ruleopt =rule->getOptionsObject();
+
+    if (target=="CUSTOM")
+    {
+        ostr << ruleopt->getStr("custom_str");
+        return ostr.str();
+    }
+
+    if ( compiler->getCachedFwOpt()->getBool("use_ULOG") &&
+         target=="LOG") target="ULOG";
+    
     ostr << " -j " << target << " ";
 
     if (target=="REJECT")
@@ -202,6 +265,40 @@ string PolicyCompiler_ipt::PrintRule::_printTarget(PolicyRule *rule)
     if (target=="LOG" || target=="ULOG")    
         ostr << _printLogParameters(rule);
     
+    if (target=="MARK")
+    {
+        ostr << " --set-mark " << ruleopt->getStr("tagvalue");
+    }
+
+    if (target=="CONNMARK")
+    {
+        ostr << ruleopt->getStr("CONNMARK_arg");
+    }
+
+    if (target=="CLASSIFY")
+    {
+        ostr << " --set-class " << ruleopt->getStr("classify_str");
+    }
+
+    if (target=="ROUTE")
+    {
+        string a;
+        a = ruleopt->getStr("ipt_iif");
+        if (!a.empty()) ostr << " --iif " << a;
+
+        a = ruleopt->getStr("ipt_oif");
+        if (!a.empty()) ostr << " --oif " << a;
+
+        a = ruleopt->getStr("ipt_gw");
+        if (!a.empty()) ostr << " --gw " << a;
+
+        bool c = ruleopt->getBool("ipt_continue");
+        if (c) ostr << " --continue";
+
+        c = ruleopt->getBool("ipt_tee");
+        if (c) ostr << " --tee";
+    }
+
     return ostr.str();
 }
 
@@ -229,13 +326,28 @@ string PolicyCompiler_ipt::PrintRule::_printDirectionAndInterface(PolicyRule *ru
     string::size_type n;
     if ( (n=iface_name.find("*"))!=string::npos)    iface_name[n]='+';
 
-//    Interface *rule_iface =  compiler->getCachedFwInterface(iface_id);
+    string version=compiler->fw->getStr("version");
 
-    if (rule->getDirection()==PolicyRule::Inbound)   
-        ostr << " -i "  << iface_name; 
+    Interface *rule_iface =
+        compiler->getCachedFwInterface(rule->getInterfaceId());
 
-    if (rule->getDirection()==PolicyRule::Outbound)  
-        ostr << " -o "  << iface_name; 
+    if (rule_iface->isBridgePort() && version == "1.3.0")
+    {
+        ostr << " -m physdev ";
+
+        if (rule->getDirection()==PolicyRule::Inbound)   
+            ostr << " --physdev-in "  << iface_name; 
+
+        if (rule->getDirection()==PolicyRule::Outbound)  
+            ostr << " --physdev-out "  << iface_name; 
+    } else
+    {
+        if (rule->getDirection()==PolicyRule::Inbound)   
+            ostr << " -i "  << iface_name; 
+
+        if (rule->getDirection()==PolicyRule::Outbound)  
+            ostr << " -o "  << iface_name; 
+    }
 
 //    if (rule->getDirection()==PolicyRule::Both)      
 //        compiler->output << "-i "  << rule_iface->getName() 
@@ -273,7 +385,7 @@ string PolicyCompiler_ipt::PrintRule::_printActionOnReject(libfwbuilder::PolicyR
             {
 		if (s.find("net")!=string::npos)   str << " --reject-with icmp-net-prohibited";
 		if (s.find("host")!=string::npos)  str << " --reject-with icmp-host-prohibited";
-                if (version=="1.2.9" && 
+                if ((version=="1.2.9" || version=="1.3.0") && 
                     s.find("admin")!=string::npos)  str << " --reject-with icmp-admin-prohibited";
 	    }
 	}
@@ -287,8 +399,12 @@ string PolicyCompiler_ipt::PrintRule::_printGlobalLogParameters()
     return _printLogParameters(NULL);
 }
 
-string PolicyCompiler_ipt::PrintRule::_printLogPrefix(PolicyRule *rule,
-                                                      const string &prefix)
+string PolicyCompiler_ipt::PrintRule::_printLogPrefix(int rule_num,
+						      const string &action,
+						      const string &interf,
+						      const string &chain,
+						      const string &rule_label,
+						      const string &prefix)
 {
     string s=prefix;
 
@@ -299,46 +415,52 @@ string PolicyCompiler_ipt::PrintRule::_printLogPrefix(PolicyRule *rule,
  * %C - chain name
  */
     string::size_type n;
-    if (rule && (n=s.find("%N"))!=string::npos ) 
+    if ((n=s.find("%N"))!=string::npos ) 
     {
-        std::ostringstream s1;
-        s1 << rule->getPosition();
-        s.replace(n,2,s1.str());
+      std::ostringstream s1;
+      s1 << rule_num;
+      s.replace(n,2,s1.str());
     }
-    if (rule && (n=s.find("%A"))!=string::npos ) 
+    if ((n=s.find("%A"))!=string::npos ) 
     {
-        std::ostringstream s1;
-
-        char str[64];
-        strncpy(str,rule->getStr("stored_action").c_str(),sizeof(str));
-        for (char *cptr=str; *cptr; cptr++)  *cptr=toupper(*cptr);
-
-        s.replace(n,2,str);
+      s.replace(n,2,action);
     }
-    if (rule && (n=s.find("%I"))!=string::npos ) 
+    if ((n=s.find("%I"))!=string::npos ) 
     {
-        std::ostringstream s1;
-        string rule_iface =  rule->getInterfaceStr();
-        if (rule_iface!="") {
-            s1 << rule_iface;
-            s.replace(n,2,s1.str());
-        } else
-            s.replace(n,2,"global");
+      s.replace(n,2,interf);
     }
-    if (rule && (n=s.find("%C"))!=string::npos ) 
+    if ((n=s.find("%C"))!=string::npos ) 
     {
-        s.replace(n,2,rule->getStr("ipt_chain"));
+      s.replace(n,2,chain);
     }
 
     if (s.length()>29)
     {
-        compiler->warning(_("Log prefix has been truncated to 29 characters in rule ")+rule->getLabel());
+        compiler->warning(_("Log prefix has been truncated to 29 characters in rule ")+rule_label);
         s=s.substr(0,29);
     }
 
     return _quote( s );
 }
 
+string PolicyCompiler_ipt::PrintRule::_printLogPrefix(PolicyRule *rule,
+                                                      const string &prefix)
+{
+  char action[64];
+  strncpy(action,rule->getStr("stored_action").c_str(),sizeof(action));
+  for (char *cptr=action; *cptr; cptr++)  *cptr=toupper(*cptr);
+
+  string rule_iface =  rule->getInterfaceStr();
+  if (rule_iface=="")  rule_iface = "global";
+
+  return _printLogPrefix(rule->getPosition(),
+			 action,
+			 rule_iface,
+			 rule->getStr("ipt_chain"),
+			 rule->getLabel(),
+			 prefix);
+}
+
 string PolicyCompiler_ipt::PrintRule::_printLogParameters(libfwbuilder::PolicyRule *rule)
 {
     std::ostringstream str;
@@ -431,7 +553,7 @@ string PolicyCompiler_ipt::PrintRule::_printProtocol(libfwbuilder::Service *srv)
 {
     string version=compiler->fw->getStr("version");
     string s;
-    if ( ! srv->isAny() && ! CustomService::isA(srv) ) 
+    if (! srv->isAny() && !CustomService::isA(srv)  && !TagService::isA(srv))
     {
         string pn=srv->getProtocolName();
         if (pn=="ip") pn="all";
@@ -440,7 +562,7 @@ string PolicyCompiler_ipt::PrintRule::_printProtocol(libfwbuilder::Service *srv)
 
         if (pn == "icmp")
         {
-            if (version.empty() || version=="1.2.9")
+            if (version.empty() || version=="1.2.9" || version=="1.3.0")
             {
                 s += " -m icmp ";
             }
@@ -587,8 +709,7 @@ string PolicyCompiler_ipt::PrintRule::_printSrcService(RuleElementSrv  *rel)
  * find the object. I'd rather use a cached copy in the compiler
  */
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -610,7 +731,7 @@ string PolicyCompiler_ipt::PrintRule::_printSrcService(RuleElementSrv  *rel)
 	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
 	    FWObject *o= *i;
 //	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
 	    Service *s=Service::cast( o );
 	    assert(s);
@@ -623,7 +744,7 @@ string PolicyCompiler_ipt::PrintRule::_printSrcService(RuleElementSrv  *rel)
 	if ( !str.empty() ) 
         {
             string v=compiler->fw->getStr("version");
-            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9")
+            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
                 ostr << " --sports ";
             else
                 ostr << " --source-port ";
@@ -639,8 +760,7 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
     std::ostringstream  ostr;
     FWObject *o=rel->front();
     string version=compiler->fw->getStr("version");
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -669,7 +789,7 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
 	    string str=_printICMP( ICMPService::cast(srv) );
 	    if (str.empty() ) 
             {
-                if (version.empty() || version=="1.2.9")
+                if (version.empty() || version=="1.2.9" || version=="1.3.0")
                     ostr << " --icmp-type any ";
             } else
             {
@@ -692,6 +812,11 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
 	    ostr << _printSingleObjectNegation(rel) << " "
                  << CustomService::cast(srv)->getCodeForPlatform( compiler->myPlatformName() ) << " ";
 	}
+        if (TagService::isA(srv))
+        {
+	    ostr << "-m mark --mark "
+                 << TagService::cast(srv)->getCode() << " ";
+        }
     } else 
     {
 /* use multiport */
@@ -700,7 +825,7 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
 	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
         {
 	    FWObject *o= *i;
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
 	    Service *s=Service::cast( o );
 	    assert(s);
@@ -714,7 +839,7 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
 	if ( !str.empty() ) 
         {
             string v=compiler->fw->getStr("version");
-            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9")
+            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
                 ostr << " --dports ";
             else
                 ostr << " --destination-port ";
@@ -729,6 +854,27 @@ string PolicyCompiler_ipt::PrintRule::_printAddr(Address  *o)
     PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     std::ostringstream  ostr;
 
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL)
+    {
+        if (atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+        {
+            ostr << "$" << ipt_comp->getAddressTableVarName(atrt) << " ";
+            return ostr.str();
+        }
+
+        if (atrt->getSubstitutionTypeName()==DNSName::TYPENAME)
+        {
+            return atrt->getSourceName();
+        }
+        // at this time we only support two types of MultiAddress
+        // objects: AddressTable and DNSName. Both should be converted
+        // to MultiAddressRunTime at this point. If we get some other
+        // kind of MultiAddressRunTime object, we do not know what to do
+        // with it so we stop.
+        assert(atrt==NULL);
+    }
+
     if (Interface::cast(o)!=NULL)
     {
         Interface *iface=Interface::cast(o);
@@ -859,68 +1005,37 @@ PolicyCompiler_ipt::PrintRule::PrintRule(const std::string &name) : PolicyRulePr
     init=true; 
     print_once_on_top=true;
 
-    chains["INPUT"]  =true;
-    chains["OUTPUT"] =true;
-    chains["FORWARD"]=true;
-    chains["RETURN"] =true;
-    chains["LOG"]    =true;
-    chains["ACCEPT"] =true;
-    chains["DROP"]   =true;
-    chains["REJECT"] =true;
-
+    chains["INPUT"]        =true;
+    chains["OUTPUT"]       =true;
+    chains["FORWARD"]      =true;
+    chains["PREROUTING"]   =true;
+    chains["POSTROUTING"]  =true;
+    chains["RETURN"]       =true;
+    chains["LOG"]          =true;
+    chains["ACCEPT"]       =true;
+    chains["DROP"]         =true;
+    chains["REJECT"]       =true;
+    chains["MARK"]         =true;
+    chains["CONNMARK"]     =true;
+    chains["QUEUE"]        =true;
+    chains["CLASSIFY"]     =true;
+    chains["CUSTOM"]       =true;
+    chains["ROUTE"]        =true;
 }
 
 bool  PolicyCompiler_ipt::PrintRule::processNext()
 {
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     PolicyRule         *rule    =getNext(); 
     if (rule==NULL) return false;
 
     tmp_queue.push_back(rule);
 
     compiler->output << _printRuleLabel(rule);
-
     compiler->output << _createChain(rule->getStr("ipt_chain"));
     compiler->output << _createChain(rule->getStr("ipt_target"));
-
-    string  command_line = PolicyRuleToString(rule);
-
-/* if anywhere in command_line we used variable holding an address of
- * dynamic interface (named $i_something) then we need to add
- * this command with a check for the value of this variable. We execute
- * iptables command only if the value is a non-empty string.
- */
-
-    string::size_type p1=command_line.find("$i_");
-    string  iface_name;
-    string  iface_var;
-    if ( p1!=string::npos )
-    {
-        string::size_type p2=command_line.find(" ",p1);
-        string::size_type p3=command_line.find("_",p1) +1;
-        iface_name=command_line.substr(p3,p2-p3);
-        iface_var= command_line.substr(p1,p2-p1);
-
-/* if interface name ends with '*', this is a wildcard interface. */
-        string::size_type p4;
-        if ((p4=iface_name.find("*"))!=string::npos)
-        {
-            string cmdline=command_line;
-            string iface_family_name=iface_name.substr(0,p4);
-            compiler->output << "getinterfaces " << iface_family_name << " | while read I; do" << endl;
-            compiler->output << "  ivar=`getInterfaceVarName $I`" << endl;
-            compiler->output << "  getaddr $I $ivar" << endl;
-            compiler->output << "  cmd=\"$\"$ivar"   << endl;
-            compiler->output << "  eval \"addr=$cmd\""          << endl;
-            cmdline.replace(p1,p2-p1,"$addr");
-            compiler->output << "  test -n \"$addr\" && " << cmdline;
-            compiler->output << "done" << endl;
-
-            return true;
-        } else
-            compiler->output << "test -n \"" << iface_var << "\" && ";
-    }
-
-    compiler->output << command_line;
+    compiler->output 
+        << dynamic_cast<OSConfigurator_linux24*>(compiler->osconfigurator)->printRunTimeWrappers( rule, PolicyRuleToString(rule) );
 
     return true;
 }
@@ -933,19 +1048,19 @@ string PolicyCompiler_ipt::PrintRule::PolicyRuleToString(PolicyRule *rule)
 
     RuleElementSrc *srcrel=rule->getSrc();
     ref=srcrel->front();
-    Address        *src=Address::cast(compiler->getCachedObj(ref->getStr("ref")));
+    Address        *src=Address::cast(FWReference::cast(ref)->getPointer());
     if(src==NULL)
         throw FWException(_("Broken SRC in ")+rule->getLabel());
 
     RuleElementDst *dstrel=rule->getDst();
     ref=dstrel->front();
-    Address        *dst=Address::cast(compiler->getCachedObj(ref->getStr("ref")));
+    Address        *dst=Address::cast(FWReference::cast(ref)->getPointer());
     if(dst==NULL)
         throw FWException(_("Broken DST in ")+rule->getLabel());
 
     RuleElementSrv *srvrel=rule->getSrv();
     ref=srvrel->front();
-    Service        *srv=Service::cast(compiler->getCachedObj(ref->getStr("ref")));
+    Service        *srv=Service::cast(FWReference::cast(ref)->getPointer());
     if(srv==NULL)
         throw FWException(_("Broken SRV in ")+rule->getLabel());
 
@@ -1008,7 +1123,8 @@ string PolicyCompiler_ipt::PrintRule::PolicyRuleToString(PolicyRule *rule)
 	command_line << " -m state --state NEW ";
 */
 
-    if ((!ruleopt->getBool("stateless") && rule->getStr("stored_action")=="Accept") ||
+    if ((!ruleopt->getBool("stateless") && 
+         (rule->getStr("stored_action")=="Accept" || rule->getStr("stored_action")=="Tag")) ||
          rule->getBool("force_state_check") )
     {
         /*
@@ -1019,6 +1135,8 @@ string PolicyCompiler_ipt::PrintRule::PolicyRuleToString(PolicyRule *rule)
     }
 
     command_line << _printTimeInterval(rule);
+
+    command_line << _printModules(rule);
     command_line << _printTarget(rule);
 
     command_line << _endRuleLine();
@@ -1028,33 +1146,40 @@ string PolicyCompiler_ipt::PrintRule::PolicyRuleToString(PolicyRule *rule)
     return command_line.str();
 }
 
+string PolicyCompiler_ipt::PrintRule::_declareTable()
+{
+    return "";
+}
+
 string PolicyCompiler_ipt::PrintRule::_flushAndSetDefaultPolicy()
 {
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     ostringstream res;
 
-    res << "$IPTABLES -P OUTPUT  DROP" << endl;
-    res << "$IPTABLES -P INPUT   DROP" << endl;
-    res << "$IPTABLES -P FORWARD DROP" << endl;
+//    if (ipt_comp->my_table=="filter")
+//    {
+        res << "$IPTABLES -P OUTPUT  DROP" << endl;
+        res << "$IPTABLES -P INPUT   DROP" << endl;
+        res << "$IPTABLES -P FORWARD DROP" << endl;
 
-/*
- * test if ip6tables is installed and if it works. It may be installed
- * on the system but fail because ipv6 is not compiled into the
- * kernel.
- */
-    res << "ip6tables -L -n > /dev/null 2>&1 && {"    << endl; 
-    res << "  ip6tables -P OUTPUT  DROP" << endl;
-    res << "  ip6tables -P INPUT   DROP" << endl;
-    res << "  ip6tables -P FORWARD DROP" << endl;
-    res << "}"                         << endl;
-    res << endl;
+        /*
+         * test if ip6tables is installed and if it works. It may be installed
+         * on the system but fail because ipv6 is not compiled into the
+         * kernel.
+         */
+        res << "ip6tables -L -n > /dev/null 2>&1 && {"    << endl; 
+        res << "  ip6tables -P OUTPUT  DROP" << endl;
+        res << "  ip6tables -P INPUT   DROP" << endl;
+        res << "  ip6tables -P FORWARD DROP" << endl;
+        res << "}"                         << endl;
+        res << endl;
 
 /*
  * need to flush all tables and chains before setting up any rules
  */
-    res << "\n\
+        res << "\n\
 \n\
 cat /proc/net/ip_tables_names | while read table; do\n\
-  test \"X$table\" = \"Xmangle\" && continue\n\
   $IPTABLES -t $table -L -n | while read c chain rest; do\n\
       if test \"X$c\" = \"XChain\" ; then\n\
         $IPTABLES -t $table -F $chain\n\
@@ -1063,8 +1188,8 @@ cat /proc/net/ip_tables_names | while read table; do\n\
   $IPTABLES -t $table -X\n\
 done\n\
 \n";
-    res << endl;
-
+        res << endl;
+//    }
     return res.str();
 }
 
@@ -1075,6 +1200,7 @@ string PolicyCompiler_ipt::PrintRule::_commit()
 
 string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
 {
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     ostringstream res;
 
 /*
@@ -1086,7 +1212,8 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
     string s=compiler->getCachedFwOpt()->getStr("linux24_ip_forward");
     ipforward= (s.empty() || s=="1" || s=="On" || s=="on");
 
-    if ( compiler->getCachedFwOpt()->getBool("accept_established") ) 
+    if ( compiler->getCachedFwOpt()->getBool("accept_established") &&
+         ipt_comp->my_table=="filter") 
     {
         res << _startRuleLine()
             << "INPUT   -m state --state ESTABLISHED,RELATED -j ACCEPT"
@@ -1206,9 +1333,13 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
                     << "FORWARD  -m state --state INVALID  -j drop_invalid"
                     << _endRuleLine();
 
+	    string s = compiler->getCachedFwOpt()->getStr("log_prefix");
+	    if (s.empty())
+	      s = "INVALID state -- DENY ";
+
             res << _startRuleLine()
                 << "drop_invalid  -j LOG  --log-level debug --log-prefix "
-                << _quote("INVALID state -- DENY ")
+		<< _printLogPrefix(-1,"DENY","global","drop_invalid","BLOCK INVALID",s)
                 << _endRuleLine();
 
             res << _startRuleLine()
