@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h,v 1.23 2006/08/19 21:15:20 vkurland Exp $
+  $Id: PolicyCompiler_ipt.h,v 1.24 2006/09/21 06:29:43 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -152,12 +152,12 @@ namespace fwcompiler {
         DECLARE_POLICY_RULE_PROCESSOR(InterfacePolicyRulesWithOptimization);
 
 	/**
-	 * if option "firewall is part of any" is OFF, replace all *
+	 * if option "firewall is part of any" is OFF, replace all
 	 * "Any" with "!fw" before checking for rule shadowing (if fw is
-	 * not * part of "any", then "any" does not shade the
+	 * not * part of "any", then "any" does not shadow the
 	 * firewall)
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(convertAnyToNotFW);
+        DECLARE_POLICY_RULE_PROCESSOR(convertAnyToNotFWForShadowing);
 
 	/**
 	 * processes rules with negation in Src if it holds only one object
@@ -178,24 +178,87 @@ namespace fwcompiler {
 
 	/**
 	 * processes rules with negation in Src
+         *
+         * Argument dm defines mode of operation for this rule processor:
+         *   if it is false, processor compiles the rule
+         *   if it is true, it works in the mode of shadowing detection
+         *
+         * difference is that in shadowing detection mode it does not
+         * replace objects in dst,srv and time with any so that we can
+         * properly check shadowing later. Regular rule processor that
+         * deals with negation in SRC replaces objects in rule
+         * elements DST, SRV and Time with any which causes problems
+         * because these rule elements then match those in other
+         * rules, but they really should not match them because
+         * originally they had specific object so only some packets
+         * would match these rules.
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(SrcNegation);
 
+        class SrcNegation : public PolicyRuleProcessor
+        {
+            bool shadowing_mode;
+            public:
+            SrcNegation(bool dm,
+                        const std::string &name) : PolicyRuleProcessor(name)
+            {
+                shadowing_mode = dm;
+            }
+            virtual bool processNext();
+        };
+
+        
 	/**
 	 * processes rules with negation in Dst
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(DstNegation);
 
+        class DstNegation : public PolicyRuleProcessor
+        {
+            bool shadowing_mode;
+            public:
+            DstNegation(bool dm,
+                        const std::string &name) : PolicyRuleProcessor(name)
+            {
+                shadowing_mode = dm;
+            }
+            virtual bool processNext();
+        };
+
+        
 	/**
 	 * processes rules with negation in Srv
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(SrvNegation);
 
+        class SrvNegation : public PolicyRuleProcessor
+        {
+            bool shadowing_mode;
+            public:
+            SrvNegation(bool dm,
+                        const std::string &name) : PolicyRuleProcessor(name)
+            {
+                shadowing_mode = dm;
+            }
+            virtual bool processNext();
+        };
+
+        
 	/**
 	 * processes rules with negation in Interval
 	 */
-        DECLARE_POLICY_RULE_PROCESSOR(TimeNegation);
 
+        class TimeNegation : public PolicyRuleProcessor
+        {
+            bool shadowing_mode;
+            public:
+            TimeNegation(bool dm,
+                        const std::string &name) : PolicyRuleProcessor(name)
+            {
+                shadowing_mode = dm;
+            }
+            virtual bool processNext();
+        };
+
+        
+        
         /**
 	 *   verifies combination of interface and * direction and
 	 *   fills interface and direction. After this * predicate it
@@ -256,6 +319,26 @@ namespace fwcompiler {
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(splitIfDstAny);
 
+	/**
+	 * split rule if Src==any 
+	 * 
+         * This works just like splitIfSrcAny, except is used in the
+         * part of compiler that detects rule shadowing. While
+         * compiling rules, we split the rule and set chains
+         * appropriately (one rule gets into chain OUTPUT) but leave
+         * SRC 'any' to avoid generating lots of address matches since
+         * setting chain to OUTPUT is sufficient. We can not do this
+         * while detecting shadowing and need to explicitly put the
+         * firewall object in the first of the two rules we produce.
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(splitIfSrcAnyForShadowing);
+
+        /**
+	 * split rule if Dst==any for shadowing detection. See comment
+	 * in splitIfSrcAnyForShadowing
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(splitIfDstAnyForShadowing);
+
 	/**
 	 * split rule if Src==network the firewall is connected to
 	 * 
