@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf_writers.cpp,v 1.20 2006/09/17 18:58:13 vkurland Exp $
+  $Id: PolicyCompiler_pf_writers.cpp,v 1.21 2006/10/15 23:54:02 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -119,7 +119,11 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
         compiler->output << "anchor " << ruleopt->getStr("branch_name") << " ";
         break;
     default:
-        compiler->output << rule->getActionAsString() << " ";
+        compiler->abort(
+            string("Unknown action '") + rule->getActionAsString()
+            + "' in rule " + rule->getLabel()
+        );
+//        compiler->output << rule->getActionAsString() << " ";
     }
 }
 
